@@ -18,7 +18,6 @@ from __future__ import print_function
 import copy
 import functools
 import os
-import six
 import sys
 
 from oslo_utils import encodeutils
@@ -29,8 +28,9 @@ from glanceclient.common import utils
 from glanceclient import exc
 import glanceclient.v1.images
 
-CONTAINER_FORMATS = 'Acceptable formats: ami, ari, aki, bare, and ovf.'
-DISK_FORMATS = ('Acceptable formats: ami, ari, aki, vhd, vmdk, raw, '
+CONTAINER_FORMATS = ('Acceptable formats: ami, ari, aki, bare, ovf, ova,'
+                     'docker.')
+DISK_FORMATS = ('Acceptable formats: ami, ari, aki, vhd, vdhx, vmdk, raw, '
                 'qcow2, vdi, iso, and ploop.')
 DATA_FIELDS = ('location', 'copy_from', 'file')
 
@@ -128,7 +128,7 @@ def _image_show(image, human_readable=False, max_column_width=80):
     info = copy.deepcopy(image._info)
     if human_readable:
         info['size'] = utils.make_size_human_readable(info['size'])
-    for (k, v) in six.iteritems(info.pop('properties')):
+    for (k, v) in info.pop('properties').items():
         info['Property \'%s\'' % k] = v
 
     utils.print_dict(info, max_column_width=max_column_width)
